/*******************************************************************************
* File Name: main.c
*
* Version: 1.0
*
* Description:
*  This is the source code for the project which demonstrates BLE Privacy on
*  the Central side. The Central resolves a private address of the Peripheral.
*
* Hardware Dependency:
*  CY8CKIT-042-BLE Bluetooth Low Energy (BLE) Pioneer Kit
*
********************************************************************************
* Copyright (2015), Cypress Semiconductor Corporation.
******************************************************************************
* This software is owned by Cypress Semiconductor Corporation (Cypress) and is
* protected by and subject to worldwide patent protection (United States and
* foreign), United States copyright laws and international treaty provisions.
* Cypress hereby grants to licensee a personal, non-exclusive, non-transferable
* license to copy, use, modify, create derivative works of, and compile the
* Cypress Source Code and derivative works for the sole purpose of creating
* custom software in support of licensee product to be used only in conjunction
* with a Cypress integrated circuit as specified in the applicable agreement.
* Any reproduction, modification, translation, compilation, or representation of
* this software except as specified above is prohibited without the express
* written permission of Cypress.
*
* Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
* REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
* Cypress reserves the right to make changes without further notice to the
* materials described herein. Cypress does not assume any liability arising out
* of the application or use of any product or circuit described herein. Cypress
* does not authorize its products for use as critical components in life-support
* systems where a malfunction or failure may reasonably be expected to result in
* significant injury to the user. The inclusion of Cypress' product in a life-
* support systems application implies that the manufacturer assumes all risk of
* such use and in doing so indemnifies Cypress against all charges. Use may be
* limited by and subject to the applicable Cypress software license agreement.
*******************************************************************************/


/*******************************************************************************
* Included headers
*******************************************************************************/
#include <project.h>
#include <stdbool.h>
#include <common.h>


/*******************************************************************************
* Enumerations
*******************************************************************************/
typedef enum
{
    AUTHENTICATION_NOT_CONNECTED,
    AUTHENTICATION_COMPLETE_BONDING_REQD,
    AUTHENTICATION_BONDING_COMPLETE,
    AUTHENTICATION_BONDING_REMOVE_WAITING_EVENT,
    AUTHENTICATION_BONDING_REMOVE_GO_AHEAD
} AUTHENTICATION_STATE;

typedef enum PowerMode_t {
    WAKEUP_SLEEP,
    WAKEUP_DEEPSLEEP,
    ACTIVE,
    SLEEP,
    DEEPSLEEP
} PowerMode_t;
/*******************************************************************************
* Structures
*******************************************************************************/

#define DEBONDING               (1u)
#define NOTIFY          (2u)
#define ManageBlePower()            CyBle_EnterLPM(CYBLE_BLESS_DEEPSLEEP)
#define CAPACITOR_TRIM_VALUE       0x00004355
uint32 WDT_COUNT0_MATCH=0x7FFFu;

CY_ISR_PROTO(WdtIsrHandler);
#ifdef WAKEUP_ENABLE
CY_ISR_PROTO(WakeupIsrHandler);
#endif
/*******************************************************************************
* Global variables
*******************************************************************************/
AUTHENTICATION_STATE authState = AUTHENTICATION_NOT_CONNECTED;

CYBLE_GAP_BONDED_DEV_ADDR_LIST_T bondedDeviceList;
static CYBLE_GAP_BD_ADDR_T      peerAddr;
uint8 bondedDeviceBdHandle;
uint8 connectionIrk[CYBLE_GAP_SMP_IRK_SIZE] = {0};
uint8 userStoppedScan = 0;
bool irkPresentFlag = false;
bool resolveAddressFlag = false;
volatile static bool peerDeviceFound         = false;
volatile static bool notificationEnabled     = false;

uint32 lowPowerFlag;
uint8  LEDFLAG=0;
uint16 txCharHandle             = 0;                /* Handle for the TX data characteristic */
uint16 txCharDescHandle         = 0;                /* Handle for the TX data characteristic descriptor */
uint16 bleUartServiceHandle     = 0;                /* Handle for the BLE UART service */
uint16 bleUartServiceEndHandle  = 0;                /* End handle for the BLE UART service */
uint16 mtuSize                  = CYBLE_GATT_MTU;   /* MTU size to be used by Client and Server after MTU exchange */
const uint8 enableNotificationParam[2] = {0x01, 0x00};

static INFO_EXCHANGE_STATE_T    infoExchangeState   = INFO_EXCHANGE_START;

CYBLE_GATT_ATTR_HANDLE_RANGE_T  attrHandleRange;
CYBLE_GATTC_FIND_INFO_REQ_T     charDescHandleRange;
PowerMode_t applicationPower;
/* UUID of the custom BLE UART service */
const uint8 bleUartServiceUuid[16]    = {
                                            0x31, 0x01, 0x9b, 0x5f, 0x80, 0x00, 0x00,0x80, \
                                            0x00, 0x10, 0x00, 0x00, 0xd0, 0xcd, 0x03, 0x00 \
                                        };

/* UUID of the TX attribute of the custom BLE UART service */
const uint8 uartTxAttrUuid[16]        = {
                                            0x31, 0x01, 0x9b, 0x5f, 0x80, 0x00, 0x00,0x80, \
                                            0x00, 0x10, 0x00, 0x00, 0xd1, 0xcd, 0x03, 0x00 \
                                        };
/* structure to be passed for discovering service by UUID */
const CYBLE_GATT_VALUE_T    bleUartServiceUuidInfo = { 
                                                        (uint8 *) bleUartServiceUuid, \
                                                        CYBLE_GATT_128_BIT_UUID_SIZE,\
                                                        CYBLE_GATT_128_BIT_UUID_SIZE \
                                                      };

/* structure to be passed for the enable notification request */
CYBLE_GATTC_WRITE_REQ_T     enableNotificationReqParam   = {
                                                                {(uint8*)enableNotificationParam, 2, 2},
                                                                0
                                                            };
/*******************************************************************************
* Function definitions
*******************************************************************************/


/*******************************************************************************
* Function Name: StartScan()
********************************************************************************
* Summary:
* Starts a scan on the Central device.
*
* Parameters:
* None
*
* Return:
* None
*
* Theory:
* The function starts BLE scan and resets scan related variables.
*
* Side Effects:
* None
*
*******************************************************************************/
static void StartScan(void)
{
    /* Re-initialize all variables for scanning and then start a fresh scan */
    userStoppedScan = 0;
    
    CyBle_GapcStartScan(CYBLE_SCANNING_FAST);
    UART_UartPutString("\n\rList of devices: ");
}
void HandleBleProcessing(void)
{    
    CYBLE_API_RESULT_T      cyble_api_result;
    
    if(peerDeviceFound)
    {
        UART_UartPutString("peer device found,stop scan.");
        userStoppedScan =true;
        CyBle_GapcStopScan();
        while(userStoppedScan == true)
        {
            CyBle_ProcessEvents();
        }
        
        cyble_api_result = CyBle_GapcConnectDevice(&peerAddr);
        
	    if(CYBLE_ERROR_OK == cyble_api_result)
	    {
		    peerDeviceFound = false;
            UART_UartPutString("connected with device success!");
	    }
    }

    switch (cyBle_state)
    {
        case CYBLE_STATE_INITIALIZING:

        break;
        case CYBLE_STATE_SCANNING:
                                   
            break;
        
        case CYBLE_STATE_CONNECTED:                
            
            /* if Client does not has all the information about attribute handles 
             * call procedure for getting it */
            if((INFO_EXCHANGE_COMPLETE != infoExchangeState))
            {
                attrHandleInit();
            }
            /* enable notifications if not enabled already */
            else if(false == notificationEnabled)
            {             
                enableNotifications();   
            }

        break;
               
        case CYBLE_STATE_DISCONNECTED:
        {
            if((!peerDeviceFound)&&(CyBle_GetState()!=CYBLE_STATE_SCANNING))
            {               
                StartScan();
                CyBle_ProcessEvents();
                UART_UartPutString("CYBLE_STATE_DISCONNECTED, start scan again");
            }
            break;
            
        }
        
        default:
            break;       
    }
}

/*******************************************************************************
* Function Name: ResolvePrivateAddress()
********************************************************************************
* Summary:
* Tries to resolve a private address with a given IRK.
*
* Parameters:
* None
*
* Return:
* None
*
* Theory:
* If an IRK (Identity Resolving Key) is present, the function tries to resolve
* a private address with this key. If the address is resolved, it then updates
* the bond information with this.
*
* Side Effects:
* None
*
*******************************************************************************/
static void ResolvePrivateAddress(void)
{
    uint8 myIrk[16];
    uint32 apiReturn;
    uint8 irkcounter;
    memcpy(myIrk, connectionIrk, CYBLE_GAP_SMP_IRK_SIZE);
    UART_UartPutString("IRK IS:");
    for(irkcounter=0;irkcounter<17;irkcounter++)
    {
    UART_UartPutChar(HexToAscii(myIrk[irkcounter],0));
    UART_UartPutChar(HexToAscii(myIrk[irkcounter],1));
    }
    /* Random address; see if it can be resolved */
    if(irkPresentFlag == true)
    {
        apiReturn = CyBle_GapcResolveDevice(peerAddr.bdAddr, myIrk);        
        if(apiReturn == CYBLE_ERROR_OK)
        {
            UART_UartPutString(" Resolved. ");
            /* Update bonding information for the device */
            if(CYBLE_ERROR_OK == CyBle_GapcSetRemoteAddr(bondedDeviceBdHandle, peerAddr))
            {
                /* Store to flash; don't stop scanning for this */
                while(CYBLE_ERROR_OK != CyBle_StoreBondingData(1));
                UART_UartPutString("Updated bonding info. ");
            }
        }
        else
        {
            UART_UartPutString(" Not Resolved. IRK doesn't match. Return code: 0x");
            UART_UartPutChar(HexToAscii(apiReturn, 1));
            UART_UartPutChar(HexToAscii(apiReturn, 0));
            peerDeviceFound=0;
            LEDFLAG=1;
            
        }
    }
    else
    {
        UART_UartPutString(" Cannot Resolve. No IRK stored. ");
        peerDeviceFound=0;
        LEDFLAG=1;
    }
}


/*******************************************************************************
* Function Name: StackEventHandler()
********************************************************************************
* Summary:
* Event handler function for the BLE events processing.
*
* Parameters:
* uint32 eventCode: The event to be processed
* void * eventParam: Pointer to hold the additional information associated 
*                    with an event
*
* Return:
* None
*
* Theory:
* The function is responsible for handling the events generated by the stack.
* The function scans for devices and resolves a private address.
*
* Side Effects:
* None
*
*******************************************************************************/
void StackEventHandler(uint32 event, void * eventParam)
{
    CYBLE_GAP_SMP_KEY_DIST_T * keys;
    CYBLE_GATTC_READ_BY_TYPE_RSP_PARAM_T    *readResponse;
    CYBLE_GAPC_ADV_REPORT_T		            *advReport;
    CYBLE_GATTC_FIND_BY_TYPE_RSP_PARAM_T    *findResponse;
    CYBLE_GATTC_FIND_INFO_RSP_PARAM_T       *findInfoResponse;   
    CYBLE_BLESS_CLK_CFG_PARAMS_T clockConfig;
    uint8 counter;
    
    switch(event)
    {
        case CYBLE_EVT_STACK_ON:
            /* load capacitors on the ECO should be tuned and the tuned value
            ** must be set in the CY_SYS_XTAL_BLERD_BB_XO_CAPTRIM_REG  */
            CY_SYS_XTAL_BLERD_BB_XO_CAPTRIM_REG = CAPACITOR_TRIM_VALUE;    
        
            /* Get the configured clock parameters for BLE sub-system */
            CyBle_GetBleClockCfgParam(&clockConfig);    
            
            /* Update the sleep clock inaccuracy PPM based on WCO crystal used */
            /* If you see frequent link disconnection, tune your WCO or update the sleep clock accuracy here */
            clockConfig.bleLlSca =   CYBLE_LL_SCA_000_TO_020_PPM;
            
            /* set the clock configuration parameter of BLE sub-system with updated values*/
            CyBle_SetBleClockCfgParam(&clockConfig);
            /* Stack initialized; start scanning */
            UART_UartPutString("Scanning. \n\r");
            UART_UartPutString("Press 'C' followed by the device number when you want to connect to that device. \n\r");
            UART_UartPutString("Press 'D' to disconnect from a connected peer device. \n\r");
            UART_UartPutString("Press 'S' to refresh the scan list. \n\r");
            UART_UartPutString("Press 'R' to clear the bond list. \n\r");
            StartScan();
            LEDFLAG=1;
            break;
            
        
        case CYBLE_EVT_GAPC_SCAN_PROGRESS_RESULT:
            /* A new device listed in the scan report */
            advReport = (CYBLE_GAPC_ADV_REPORT_T *)eventParam;
            /* check if report has manfacturing data corresponding to the intended matching peer */
            if((advReport->eventType == CYBLE_GAPC_SCAN_RSP) && (advReport->dataLen == 0x06) \
                    && (advReport->data[1] == 0xff) && (advReport->data[2] == 0x31)  \
                    && (advReport->data[3] == 0x01) && (advReport->data[4] == 0x3b) \
                    && (advReport->data[5] == 0x04))
            {
                UART_UartPutString("device found!");
                peerDeviceFound = true;
                /* Store information for a maximum of 10 devices */

                    UART_UartPutString("\n\r");
                    CyBle_GapGetBondedDevicesList(&bondedDeviceList);
                    if(bondedDeviceList.count != 0)
                    {
                        UART_UartPutString("bonded list found");
                    }
                    if(bondedDeviceList.count==0)
                    {
                        UART_UartPutString("not bonded");
                    }
                    
                    if(advReport->peerAddrType == CYBLE_GAP_ADDR_TYPE_RANDOM)
                    {
                        UART_UartPutString(". Random Address: ");
                        resolveAddressFlag = true;
                    }
                    else
                    {
                        UART_UartPutString(". Public Address: ");
                    }
                    
                    for(counter = 6; counter > 0; counter--)
                    {
                        UART_UartPutChar(HexToAscii(advReport->peerBdAddr[counter - 1], 1));
                        UART_UartPutChar(HexToAscii(advReport->peerBdAddr[counter - 1], 0));
                        UART_UartPutChar(' ');
                    }
                    memcpy(peerAddr.bdAddr, advReport->peerBdAddr, sizeof(peerAddr.bdAddr));
                    peerAddr.type = advReport->peerAddrType; 

            }
            break;
        
        
        case CYBLE_EVT_GAPC_SCAN_START_STOP:
            
            if(authState != AUTHENTICATION_BONDING_REMOVE_WAITING_EVENT)
            {
                if(CyBle_GetState() == CYBLE_STATE_DISCONNECTED)
                {
                    UART_UartPutString("\n\n\rEnter start stop mode ");
                    if(userStoppedScan == true)
                    {
                        /* Scan stopped manually; do not restart scan */
                        userStoppedScan = false;
                        UART_UartPutString("\n\n\r userStoppedScan == true");
                    }
                    else
                    {
                        /* Scanning timed out; Restart scan */
                        UART_UartPutString("\n\n\rRestarting scan. ");
//                        StartScan();
                        LED_Write(0);
                        CySysPmHibernate();
                    }
                }
            }
            else
            {
                /* User stopped scan for bonding purposes */
                UART_UartPutString("Scan stopped. ");
                authState = AUTHENTICATION_BONDING_REMOVE_GO_AHEAD;
            }
            break;
  
        case CYBLE_EVT_GAP_KEYINFO_EXCHNGE_CMPLT:
            /* Copy the IRK key to a buffer; to be used for resolving a 
             * private address.
             */
            UART_UartPutString("key info exchange");
            keys = (CYBLE_GAP_SMP_KEY_DIST_T *) eventParam;
            memcpy(connectionIrk, keys->irkInfo, CYBLE_GAP_SMP_IRK_SIZE);
            irkPresentFlag = true;
            break;
            
            
        case CYBLE_EVT_GAP_AUTH_COMPLETE:
            /* Authentication complete; initiate bonding */
            UART_UartPutString("\n\rAuthentication complete.");
            authState = AUTHENTICATION_COMPLETE_BONDING_REQD;
            LEDFLAG=0;
            LED_Write(0);
            break;
            
            
        case CYBLE_EVT_GAP_AUTH_FAILED:
            /* Authentication failed */
            UART_UartPutString("\n\rAuthentication failed. Wait for central debonding");

            peerDeviceFound=0;
            /* Update authentication state so that bond information could be 
             * removed later 
            */
            authState = AUTHENTICATION_BONDING_COMPLETE;
            LEDFLAG=2;
            break;
           
        case CYBLE_EVT_GAP_DEVICE_CONNECTED:
            UART_UartPutString("\n\rConnected. ");
            LEDFLAG=0;
            /* Cache the BD handle to resolve private addresses later */
            bondedDeviceBdHandle = cyBle_connHandle.bdHandle;
            BRAKE_Write(1);
            break;
            
            
        case CYBLE_EVT_GAP_DEVICE_DISCONNECTED:

            notificationEnabled     = false;
            infoExchangeState       = INFO_EXCHANGE_START;
            UART_UartPutString("\n\n\rDisconnected. ");
            LEDFLAG=1;
            BRAKE_Write(0);
            if(authState != AUTHENTICATION_BONDING_REMOVE_WAITING_EVENT)
            {
                /* Disconnected; restart scan. */
                UART_UartPutString("Scanning again.");
//                StartScan();
            }
            else
            {
                authState = AUTHENTICATION_BONDING_REMOVE_GO_AHEAD;
                UART_UartPutString("GO TO authstate AUTHENTICATION_BONDING_REMOVE_GO_AHEAD");
            }
            break;
            
        case CYBLE_EVT_GATTC_READ_BY_TYPE_RSP:
            
            readResponse = (CYBLE_GATTC_READ_BY_TYPE_RSP_PARAM_T *) eventParam;
            
            if(0 == memcmp((uint8 *)&(readResponse->attrData.attrValue[5]), (uint8 *)uartTxAttrUuid, 16))
            {
                txCharHandle = readResponse->attrData.attrValue[3];
                txCharHandle |= (readResponse->attrData.attrValue[4] << 8);
                
                infoExchangeState |= TX_ATTR_HANDLE_FOUND;
            }
            infoExchangeState |= RX_ATTR_HANDLE_FOUND;           
            break;
            
        case CYBLE_EVT_GATTC_FIND_INFO_RSP:
            
            findInfoResponse = (CYBLE_GATTC_FIND_INFO_RSP_PARAM_T *) eventParam;
            
            if((0x29 == findInfoResponse->handleValueList.list[3]) && \
                                (0x02 == findInfoResponse->handleValueList.list[2]))
            {
                txCharDescHandle = findInfoResponse->handleValueList.list[0];
                txCharDescHandle |= findInfoResponse->handleValueList.list[1] << 8;
            
                infoExchangeState |= TX_CCCD_HANDLE_FOUND;
            }
           
            break;
        case CYBLE_EVT_GATTC_XCHNG_MTU_RSP:   
           
            /*set the 'mtuSize' variable based on the minimum MTU supported by both devices */
            if(CYBLE_GATT_MTU > ((CYBLE_GATT_XCHG_MTU_PARAM_T *)eventParam)->mtu)
            {
                mtuSize = ((CYBLE_GATT_XCHG_MTU_PARAM_T *)eventParam)->mtu;
            }
            else
            {
                mtuSize = CYBLE_GATT_MTU;
            }
            
            infoExchangeState |= MTU_XCHNG_COMPLETE;
            
            break;
            
        case CYBLE_EVT_GATTC_HANDLE_VALUE_NTF:
            
            HandleUartRxTraffic((CYBLE_GATTC_HANDLE_VALUE_NTF_PARAM_T *)eventParam);
			
            break;
            
        case CYBLE_EVT_GATTC_FIND_BY_TYPE_VALUE_RSP:
            
            findResponse            = (CYBLE_GATTC_FIND_BY_TYPE_RSP_PARAM_T *) eventParam;
            
            bleUartServiceHandle    = findResponse->range->startHandle;
            bleUartServiceEndHandle = findResponse->range->endHandle;
            
            infoExchangeState |= BLE_UART_SERVICE_HANDLE_FOUND;
            
            break;
        case CYBLE_EVT_GATTS_XCNHG_MTU_REQ:
            
            /*set the 'mtuSize' variable based on the minimum MTU supported by both devices */
            if(CYBLE_GATT_MTU > ((CYBLE_GATT_XCHG_MTU_PARAM_T *)eventParam)->mtu)
            {
                mtuSize = ((CYBLE_GATT_XCHG_MTU_PARAM_T *)eventParam)->mtu;
            }
            else
            {
                mtuSize = CYBLE_GATT_MTU;
            }
            
            break;    
        case CYBLE_EVT_GATTC_WRITE_RSP:
            
            notificationEnabled = true;
            
            #ifdef PRINT_MESSAGE_LOG   
                UART_UartPutString("\n\rNotifications enabled\n\r");
                UART_UartPutString("\n\rStart entering data:\n\r");
            #endif
            
            break;
                 
        default:
            break;
    }
}
__inline void ManageSystemPower()
{
    /* Variable declarations */
    CYBLE_BLESS_STATE_T blePower;
    uint8 interruptStatus ;
    
   /* Disable global interrupts to avoid any other tasks from interrupting this section of code*/
    interruptStatus  = CyEnterCriticalSection();
    
    /* Get current state of BLE sub system to check if it has successfully entered deep sleep mode */
    blePower = CyBle_GetBleSsState();
    
    /* System can enter DeepSleep only when BLESS and rest of the application are in DeepSleep 
     * power modes */
    if((blePower == CYBLE_BLESS_STATE_DEEPSLEEP || blePower == CYBLE_BLESS_STATE_ECO_ON) && 
        applicationPower == DEEPSLEEP)
    {
        applicationPower = WAKEUP_DEEPSLEEP; 
        CySysPmDeepSleep(); 
    }
    else if((blePower != CYBLE_BLESS_STATE_EVENT_CLOSE))
    {
        if(applicationPower == DEEPSLEEP)
        {
            applicationPower = WAKEUP_DEEPSLEEP;
            
            /* change HF clock source from IMO to ECO, as IMO is not required and can be stopped to save power */
            CySysClkWriteHfclkDirect(CY_SYS_CLK_HFCLK_ECO); 
            /* stop IMO for reducing power consumption */
            CySysClkImoStop();              
            /* put the CPU to sleep */
            CySysPmSleep();            
            /* starts execution after waking up, start IMO */
            CySysClkImoStart();
            /* change HF clock source back to IMO */
            CySysClkWriteHfclkDirect(CY_SYS_CLK_HFCLK_IMO);
			
        }
        else if(applicationPower == SLEEP )
        {
            /* If the application requires IMO for its operation, we should not switch the HFCLK source */
            applicationPower = WAKEUP_SLEEP;        
            CySysPmSleep();            
            
        }
    }
    
    /* Enable interrupts */
    CyExitCriticalSection(interruptStatus );

}

/*******************************************************************************
* Function Name: ManageApplicationPower()
********************************************************************************
*
* Summary:
*   This function puts the application componnents in appropriate low power modes 
*   based on the state of the components.
*
* Parameters:
*  none
*
********************************************************************************/
__inline void ManageApplicationPower()
{
    switch(applicationPower)
    {
        case ACTIVE: // dont need to do anything
        break;
        
        case WAKEUP_SLEEP: // do whatever wakeup needs to be done - probably never do anything
        
            applicationPower = ACTIVE;
        break;
        
        case WAKEUP_DEEPSLEEP: // do whatever wakeup needs to be done.
        
            applicationPower = ACTIVE;
        break;
        
        case SLEEP: 
        /** Add code to place the application components to sleep here  */
        break;
        
        case DEEPSLEEP:
        /** Add code to place the application components to deep sleep here  */
        
        break;
    }
}

/*******************************************************************************
* Function Name: RunApplication()
********************************************************************************
*
* Summary:
*   This function runs application specific code. this is a template. This function
*   may be replaced by customer specific functions. 
*
* Parameters:
*  none
*
********************************************************************************/
__inline void RunApplication()
{
    /** Place your application code here **/
    
    /* Once application is done with all processing, enter low-power mode */
    if(0) /* if you are ready to sleep then set it up to go to sleep */
    {
        applicationPower = SLEEP;
    }
    
    if(1) /* if you are done with everything, then deepsleep */
    {
        applicationPower = DEEPSLEEP;
    }
}

/*******************************************************************************
* Function Name: main()
********************************************************************************
* Summary:
* The top level application function.
*
* Parameters:
* None
*
* Return:
* None
*
* Theory:
* Initializes the BLE and UART components and then process BLE events regularly.
* Checks whether an IRK is stored in the bond information of this device and 
* loads it (if available) for private address resolution.
* It implements bonding and privacy (resolvable private address resolution) 
* feature.
* It also handles UART commands for connecting to a device, disconnecting from
* a device, refreshing scan, and clearing the bonded devices' list.
*
* For more information on privacy feature, refer to Bluetooth 4.1 Specification, 
* Volume 3, Part C, Section 10.7.
*
* Side Effects:
* None
*
*******************************************************************************/
int main()
{
    uint8 command;
    uint8 counter;
    CYBLE_GAP_BD_ADDR_T clearAllDevices = {{0,0,0,0,0,0},0};
    CYBLE_GAP_BD_ADDR_T bondedDeviceAddress;
    uint8 bondedDeviceKeysFlag;
    CYBLE_API_RESULT_T      cyble_api_result;
    
    #ifdef WAKEUP_ENABLE
        
    isr_WakeupPin_StartEx(WakeupIsrHandler);

    /* Wait till the switch is pressed */
    while (Wakeup_Pin_Read() == 0u);//防抖
    CyDelay(100u);

    Wakeup_Pin_ClearInterrupt();
    isr_WakeupPin_ClearPending();
    
    #endif
    WdtIsr_StartEx(WdtIsrHandler);
    CySysWdtWriteMode(CY_SYS_WDT_COUNTER0, CY_SYS_WDT_MODE_INT);
	CySysWdtWriteMatch(CY_SYS_WDT_COUNTER0, WDT_COUNT0_MATCH);
	CySysWdtWriteClearOnMatch(CY_SYS_WDT_COUNTER0, 1u);
    CySysWdtEnable(CY_SYS_WDT_COUNTER0_MASK);
    CySysWdtLock();
	CySysWdtDisable(CY_SYS_WDT_COUNTER0_MASK);
	CySysWdtUnlock();
    
    /* Enable global interrupts */
    CyGlobalIntEnable; 
    
    /* Start BLE and UART components for the project */
    CyBle_Start(StackEventHandler);
    UART_Start();

    /* Clear screen and put a welcome message */
    UART_UartPutChar(12);
    UART_UartPutString("========= BLE Privacy Demo - Central =========\n\n\r");
    /* Internal low power oscillator is stopped as it is not used in this project */
    CySysClkIloStop();
    
    /* Set the divider for ECO, ECO will be used as source when IMO is switched off to save power */
    CySysClkWriteEcoDiv(CY_SYS_CLK_ECO_DIV8);
    
    /* Find out whether the device has bonded information stored already or not */
    CyBle_GapGetBondedDevicesList(&bondedDeviceList);
    if(bondedDeviceList.count != 0)
    {
        CYBLE_GAP_SMP_KEY_DIST_T securityKeys;
        
        /* Take the last bonded device */
        memcpy(bondedDeviceAddress.bdAddr, bondedDeviceList.bdAddrList[0].bdAddr, 6);
        bondedDeviceAddress.type = bondedDeviceList.bdAddrList[0].type;
        
        /* Get the BD handle of that bonded device */
        CyBle_GapGetPeerBdHandle(&bondedDeviceBdHandle, &bondedDeviceAddress);
        
        /* Get the IRK for that bonded device */
        bondedDeviceKeysFlag |= 0x01;   /* Identity Resolution Key */ 
        CyBle_GapGetPeerDevSecurityKeyInfo(bondedDeviceBdHandle, &bondedDeviceKeysFlag, &securityKeys);
        memcpy(connectionIrk, securityKeys.irkInfo, CYBLE_GAP_SMP_IRK_SIZE);
        
        /* Ready to resolve private address */
        irkPresentFlag = true;
        authState = AUTHENTICATION_BONDING_COMPLETE;
    }
    else
    {
        /* Cannot resolve private address */
        irkPresentFlag = false;
        authState = AUTHENTICATION_NOT_CONNECTED;
    }
    
    /* Infinite loop */
    while(1)
    {

        ManageBlePower();
            RunApplication();
//        }
        
        /***********************************************************************
        *  Process application power modes
        ***********************************************************************/
        ManageApplicationPower();   
        
        /***********************************************************************
        *  Manage system power mode based on application and BLE power modes
        ***********************************************************************/
        ManageSystemPower();        
        /* Resolve a private address */
        if(resolveAddressFlag == true)
        {
            ResolvePrivateAddress();
            resolveAddressFlag = false;
        }

        /* Handle the state machine for authentication and bonding */
        switch(authState)
        {
            case AUTHENTICATION_COMPLETE_BONDING_REQD:
                /* Store bonding data of the current connection */
                while(CYBLE_ERROR_OK != CyBle_StoreBondingData(1));
                
                authState = AUTHENTICATION_BONDING_COMPLETE;
                UART_UartPutString("\n\rBonding complete. ");
                
                break;
                
            
            case AUTHENTICATION_BONDING_COMPLETE:
                /* See if the user pressed 'R' button to remove the bond. */
                
                if(lowPowerFlag==DEBONDING)
                {
                    LEDFLAG=0;
                    /* User wants the bond to be removed */
                    UART_UartPutString("\n\rRemove the bond. ");
                    if(CyBle_GetState() == CYBLE_STATE_CONNECTED)
                    {
                        /* Disconnect */
                        authState = AUTHENTICATION_BONDING_REMOVE_WAITING_EVENT;
                        CyBle_GapDisconnect(cyBle_connHandle.bdHandle);
                        UART_UartPutString("disconnect the peripheral");
                    }
                    else if(CyBle_GetState() == CYBLE_STATE_SCANNING)
                    {
                        /* Stop scan */
                        authState = AUTHENTICATION_BONDING_REMOVE_WAITING_EVENT;
                        CyBle_GapcStopScan();
                        UART_UartPutString("state scanning");
                    }
                    else
                    {
                        authState = AUTHENTICATION_BONDING_REMOVE_GO_AHEAD;
                        UART_UartPutString("other else");
                    }
                }
                
                break;
                
                
            case AUTHENTICATION_BONDING_REMOVE_GO_AHEAD:
                /* Remove all bonded devices in the list */
                CyBle_GapRemoveDeviceFromWhiteList(&clearAllDevices);
                
                while(CYBLE_ERROR_OK != CyBle_StoreBondingData(1));
                UART_UartPutString("Cleared the list of bonded devices. \n\r");
                
                /* Clear the IRK used for random private address resolution */
                for(counter = 0; counter < CYBLE_GAP_SMP_IRK_SIZE; counter++)
                {
                    connectionIrk[counter] = 0;
                }
                irkPresentFlag = false;
                lowPowerFlag=0;
                /* Start scanning again */
                UART_UartPutString("Scanning again.");
                
                authState = AUTHENTICATION_NOT_CONNECTED;
                break;
                
                
            default:
                break;
        }
        /* Process pending BLE events */
        HandleBleProcessing(); 
        CyBle_ProcessEvents();
    }
}

void attrHandleInit()
{
    switch(infoExchangeState)
    {
        case INFO_EXCHANGE_START:        
            CyBle_GattcDiscoverPrimaryServiceByUuid(cyBle_connHandle, bleUartServiceUuidInfo);
            break;
        
        case BLE_UART_SERVICE_HANDLE_FOUND:
            attrHandleRange.startHandle    = bleUartServiceHandle;
            attrHandleRange.endHandle      = bleUartServiceEndHandle;

            CyBle_GattcDiscoverAllCharacteristics(cyBle_connHandle, attrHandleRange);
            break;
        
        case (SERVICE_AND_CHAR_HANDLES_FOUND):
            charDescHandleRange.startHandle = txCharHandle + 1;
            charDescHandleRange.endHandle   = bleUartServiceEndHandle;

            CyBle_GattcDiscoverAllCharacteristicDescriptors(cyBle_connHandle, &charDescHandleRange);
            break;
        
        case (ALL_HANDLES_FOUND):
            CyBle_GattcExchangeMtuReq(cyBle_connHandle, CYBLE_GATT_MTU);

            break;    
            
        default:
            break;    
    }
    
    CyBle_ProcessEvents();
}

/*******************************************************************************
* Function Name: enableNotifications
********************************************************************************
*
* Summary:
*  This function enables notfications from servers. 
*
* Parameters:
*  None.
*
* Return:
*   None.
*
*******************************************************************************/
void enableNotifications()
{     
    enableNotificationReqParam.attrHandle = txCharDescHandle;   
    CyBle_GattcWriteCharacteristicDescriptors(cyBle_connHandle, (CYBLE_GATTC_WRITE_REQ_T *)(&enableNotificationReqParam));
}
void HandleUartRxTraffic(CYBLE_GATTC_HANDLE_VALUE_NTF_PARAM_T *uartRxDataNotification)
{
    uint8 led_state;
    if(uartRxDataNotification->handleValPair.attrHandle == txCharHandle)
    {
        UART_SpiUartPutArray(uartRxDataNotification->handleValPair.value.val, \
            (uint32) uartRxDataNotification->handleValPair.value.len);
        led_state=*uartRxDataNotification->handleValPair.value.val;
        if(led_state==0x31) BRAKE_Write(1);
        if(led_state==0x30) BRAKE_Write(0);
        LED_Write(~led_state);
    }
}

#ifdef WAKEUP_ENABLE
CY_ISR(WakeupIsrHandler)
{
    uint32 counter;

    /* Debounce delay */
    CyDelay(50u);

    /* If the switch is pressed for more than 1 sec, set the flag to enter Stop
     * mode else set the flag to enter hibernate mode. This flag will be cleared
     * by the device when the device wakes up from low power mode
     */
    for(counter = 0u; (counter < 20u) && (Wakeup_Pin_Read() == 0u); counter++)
    {
        CyDelay(50u);
    }
    if(counter == 20u) lowPowerFlag=DEBONDING;

    Wakeup_Pin_ClearInterrupt();
    isr_WakeupPin_ClearPending();
}
#endif
CY_ISR(WdtIsrHandler)
{
    switch(LEDFLAG)
    {
        case 0:
            LED_Write(0);//关灯
            break;
        case 1:
            LED_Write(~LED_Read());
            break;
        case 2:
            LED_Write(1);
            CyDelay(50);
            LED_Write(0);
            break;
    }

    /* Clear interrupts state */
	CySysWdtClearInterrupt(CY_SYS_WDT_COUNTER0_INT);
    WdtIsr_ClearPending();
}

/* [] END OF FILE */
