/*******************************************************************************
* File Name: main.c
*
* Version: 1.0
*
* Description:
*  This is the source code for the project which demonstrates BLE Privacy on
*  the Peripheral side. The Peripheral advertises with a private address.
*
* Hardware Dependency:
*  CY8CKIT-042-BLE Bluetooth Low Energy (BLE) Pioneer Kit
*
********************************************************************************
* Copyright (2015), Cypress Semiconductor Corporation.
******************************************************************************
* This software is owned by Cypress Semiconductor Corporation (Cypress) and is
* protected by and subject to worldwide patent protection (United States and
* foreign), United States copyright laws and international treaty provisions.
* Cypress hereby grants to licensee a personal, non-exclusive, non-transferable
* license to copy, use, modify, create derivative works of, and compile the
* Cypress Source Code and derivative works for the sole purpose of creating
* custom software in support of licensee product to be used only in conjunction
* with a Cypress integrated circuit as specified in the applicable agreement.
* Any reproduction, modification, translation, compilation, or representation of
* this software except as specified above is prohibited without the express
* written permission of Cypress.
*
* Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
* REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
* Cypress reserves the right to make changes without further notice to the
* materials described herein. Cypress does not assume any liability arising out
* of the application or use of any product or circuit described herein. Cypress
* does not authorize its products for use as critical components in life-support
* systems where a malfunction or failure may reasonably be expected to result in
* significant injury to the user. The inclusion of Cypress' product in a life-
* support systems application implies that the manufacturer assumes all risk of
* such use and in doing so indemnifies Cypress against all charges. Use may be
* limited by and subject to the applicable Cypress software license agreement.
*******************************************************************************/

/*******************************************************************************
* Included headers
*******************************************************************************/
#include <project.h>
#include <common.h>


/*******************************************************************************
* Enumerations
*******************************************************************************/
typedef enum
{
    ADVERTISEMENT_PUBLIC,
    ADVERTISEMENT_PRIVATE
} ADVERTISEMENT_STATE;

typedef enum
{
    AUTHENTICATION_NOT_CONNECTED,
    AUTHENTICATION_COMPLETE_BONDING_REQD,
    AUTHENTICATION_BONDING_COMPLETE,
    AUTHENTICATION_BONDING_REMOVE_WAITING_EVENT,
    AUTHENTICATION_BONDING_REMOVE_GO_AHEAD
} AUTHENTICATION_STATE;

typedef enum PowerMode_t {
    WAKEUP_SLEEP,
    WAKEUP_DEEPSLEEP,
    ACTIVE,
    SLEEP,
    DEEPSLEEP
} PowerMode_t;

#define ManageBlePower()            CyBle_EnterLPM(CYBLE_BLESS_DEEPSLEEP)
#define CAPACITOR_TRIM_VALUE       0x00004355
#define WDT_COUNT0_MATCH    (0x4FFFu)
#define DEBONDING               (1u)
#define NOTIFY          (2u)
#define LED_LIGHT_ON           (3u)
/*******************************************************************************
* Global variables
*******************************************************************************/
PowerMode_t applicationPower;
ADVERTISEMENT_STATE advState = ADVERTISEMENT_PUBLIC;
AUTHENTICATION_STATE authState = AUTHENTICATION_NOT_CONNECTED;
CYBLE_GAP_BONDED_DEV_ADDR_LIST_T bondedDeviceList;
CYBLE_GAP_BD_ADDR_T privateAddress;
uint8 connectionIrk[CYBLE_GAP_SMP_IRK_SIZE] = {0};

/*******************************************************************************
* Function definitions
*******************************************************************************/
uint16 notifydata[2]={0x01,0x00};
uint16 notifyenable=0;
uint16 notifyenableH=0;
uint32 lowPowerFlag;
uint8  notifyflag;
uint16 mtuSize      = CYBLE_GATT_MTU;  
/* Interrupt prototypes */
#ifdef SENSOR_ENABLE
CY_ISR_PROTO(SensorIsrHandler);
#endif
#ifdef WAKEUP_ENABLE
CY_ISR_PROTO(WakeupIsrHandler);
#endif
CY_ISR_PROTO(WdtIsrHandler);
uint8 ntf_ready = 0;
uint8 LEDFLAG=0;

/*******************************************************************************
* Function Name: AdvertisePublicly()
********************************************************************************
* Summary:
* Starts an advertisement with a public address.
*
* Parameters:
* None
*
* Return:
* None
*
* Theory:
* The function starts BLE advertisements with a public address.
*
* Side Effects:
* None
*
*******************************************************************************/
static void AdvertisePublicly(void)
{
    /* Set the original public address as the device address */
    CyBle_SetDeviceAddress(&cyBle_deviceAddress);
    
    /* Revert if any changes made to advertisement settings */
    cyBle_discoveryModeInfo.advData = &cyBle_discoveryData;
    cyBle_discoveryParam.ownAddrType = CYBLE_GAP_ADDR_TYPE_PUBLIC;
    CyBle_GappStartAdvertisement(CYBLE_ADVERTISING_FAST);

}


/*******************************************************************************
* Function Name: AdvertisePrivately()
********************************************************************************
* Summary:
* Starts an advertisement with a random resolvable-private address.
*
* Parameters:
* None
*
* Return:
* None
*
* Theory:
* The function removes the Device name from the advertisement packet which was
* configured in the BLE component wizard, and starts advertisement with a 
* random resolvable-private address.
*
* Side Effects:
* None
*
*******************************************************************************/
static void AdvertisePrivately(void)
{
    /* Create a new data structure for Advertisement data 
     * without the device name.
     */
    LEDFLAG=4;
    CYBLE_GAPP_DISC_DATA_T privateAdvData = 
    {
        {
            0x02,
            0x01,
            0x06
        },
        3
    };
    
    uint8 counter;
    
    /* Generate a new resolvable private address */
    if(CYBLE_ERROR_OK == CyBle_GapGenerateDeviceAddress(&privateAddress, CYBLE_GAP_RANDOM_PRIV_RESOLVABLE_ADDR, connectionIrk))
    {
        /* Set the new device address */
        CyBle_SetDeviceAddress(&privateAddress);
        
        /* Advertise but without the device name */
        cyBle_discoveryModeInfo.advData = &privateAdvData;
        cyBle_discoveryParam.ownAddrType = CYBLE_GAP_ADDR_TYPE_RANDOM;
        CyBle_GappStartAdvertisement(CYBLE_ADVERTISING_FAST);
        
        UART_UartPutString("Advertising with new private address: ");
        
        for(counter = 6; counter > 0; counter--)
        {
            UART_UartPutChar(HexToAscii(privateAddress.bdAddr[counter - 1], 1));
            UART_UartPutChar(HexToAscii(privateAddress.bdAddr[counter - 1], 0));
            UART_UartPutChar(' ');
        }
    }
}
void HandleUartTxTraffic(uint16 txDataClientConfigDesc)
{
    uint8   uartTxData[1]={0x30};
    uint8   uartRxData[1]={0x31};
    CYBLE_API_RESULT_T                  bleApiResult;
    CYBLE_GATTS_HANDLE_VALUE_NTF_T      uartTxDataNtf;

    if((notifyenable==1 ) && (NOTIFICATON_ENABLED == txDataClientConfigDesc)&&(lowPowerFlag!=DEBONDING))
    {
        notifyenable=0;
        uartTxDataNtf.value.val  = uartTxData;
        uartTxDataNtf.value.len  = 1;
        uartTxDataNtf.attrHandle = CYBLE_CUSTOM_SERVICE_SENSOR_VALUE_CHAR_HANDLE;
                   
        do
        {
            bleApiResult = CyBle_GattsNotification(cyBle_connHandle, &uartTxDataNtf);
            CyBle_ProcessEvents();
        }
        while((CYBLE_ERROR_OK != bleApiResult)  && (CYBLE_STATE_CONNECTED == cyBle_state));
    }
    if((notifyenableH==1 ) && (NOTIFICATON_ENABLED == txDataClientConfigDesc)&&(lowPowerFlag!=DEBONDING))
    {
        if(notifyflag)
        {
            notifyflag=0;
            notifyenableH=0;
        }
        else
        {
            notifyenableH=0;
            uartTxDataNtf.value.val  = uartRxData;
            uartTxDataNtf.value.len  = 1;
            uartTxDataNtf.attrHandle = CYBLE_CUSTOM_SERVICE_SENSOR_VALUE_CHAR_HANDLE;
                       
            do
            {
                bleApiResult = CyBle_GattsNotification(cyBle_connHandle, &uartTxDataNtf);
                CyBle_ProcessEvents();
            }
            while((CYBLE_ERROR_OK != bleApiResult)  && (CYBLE_STATE_CONNECTED == cyBle_state));
        }
    }
    
    
}

void HandleBleProcessing(void)
{    
    uint8 txDataClientConfigDesc[2];
   
    CYBLE_GATT_HANDLE_VALUE_PAIR_T  attrValue = { 
                                                    {(uint8 *)txDataClientConfigDesc, 2, 2}, 
                                                    CYBLE_CUSTOM_SERVICE_SENSOR_VALUE_CLIENT_CHARACTERISTIC_CONFIGURATION_DESC_HANDLE
                                                };

    switch (cyBle_state)
    {
        case CYBLE_STATE_ADVERTISING:
            break;
        
        case CYBLE_STATE_CONNECTED:
            
            /* read CCCD for TX characteristic for checking if notifications are enabled*/
            CyBle_GattsReadAttributeValue(&attrValue, &cyBle_connHandle, CYBLE_GATT_DB_LOCALLY_INITIATED);

            /* if stack is free, handle UART traffic */
            if(CyBle_GattGetBusStatus() != CYBLE_STACK_STATE_BUSY)
            {
                HandleUartTxTraffic((uint16)txDataClientConfigDesc[0]);
            }

            
            break;
                
        case CYBLE_STATE_DISCONNECTED:
            
            UART_UartPutString("disconnected");
            txDataClientConfigDesc[0] = NOTIFICATON_DISABLED;
            txDataClientConfigDesc[1] = NOTIFICATON_DISABLED;
            
            break;
       
        case CYBLE_STATE_INITIALIZING:
        case CYBLE_STATE_STOPPED:
        default:
            break;       
    }
}

/*******************************************************************************
* Function Name: StackEventHandler()
********************************************************************************
* Summary:
* Event handler function for the BLE events processing.
*
* Parameters:
* uint32 eventCode: The event to be processed
* void * eventParam: Pointer to hold the additional information associated 
*                    with an event
*
* Return:
* None
*
* Theory:
* The function is responsible for handling the events generated by the stack.
* The function initiates a pairing request upon connection. 
*
* Side Effects:
* None
*
*******************************************************************************/
void  StackEventHandler(uint32 event, void * eventParam)
{
    CYBLE_GAP_SMP_KEY_DIST_T * keys;
    CYBLE_GATTS_WRITE_REQ_PARAM_T   *writeReqParam;
    CYBLE_GATT_ERR_CODE_T           errorCode;
    CYBLE_BLESS_CLK_CFG_PARAMS_T clockConfig;
    uint8 counter;

    /* Common events for advertisement with public or private address */
    switch(event)
    {
        case CYBLE_EVT_STACK_ON:

            LEDFLAG=2;
            break;
    
        case CYBLE_EVT_GAP_DEVICE_CONNECTED:
            /* Initiate pairing process */
            UART_UartPutString("\n\rConnected. ");
            CyBle_GapAuthReq(cyBle_connHandle.bdHandle, &cyBle_authInfo);
            notifyflag=1;
            LEDFLAG=0;
            break;
            
  
        case CYBLE_EVT_GAP_KEYINFO_EXCHNGE_CMPLT:
            /* Copy the IRK key to a buffer; to be used for creating a 
             * private address.
             */
            UART_UartPutString("key info exchanged");
            keys = (CYBLE_GAP_SMP_KEY_DIST_T *) eventParam;
            memcpy(connectionIrk, keys->irkInfo, CYBLE_GAP_SMP_IRK_SIZE);
            break;   
            
        case CYBLE_EVT_GAP_AUTH_COMPLETE:
            /* Authentication complete; initiate bonding */
            UART_UartPutString("\n\rAuthentication complete.");
            authState = AUTHENTICATION_COMPLETE_BONDING_REQD;
            LEDFLAG=0; 
            
            break;
            
        case CYBLE_EVT_GAP_AUTH_FAILED:
            /* Authentication failed */
            UART_UartPutString("\n\rAuthentication failed. Disconnecting. ");
            CyBle_GapDisconnect(cyBle_connHandle.bdHandle);
            
            /* Update authentication state so that bond information could be 
             * removed later.
             */
            authState = AUTHENTICATION_BONDING_COMPLETE;
            break;
            
        case CYBLE_EVT_GATTS_XCNHG_MTU_REQ:
            
            if(CYBLE_GATT_MTU > ((CYBLE_GATT_XCHG_MTU_PARAM_T *)eventParam)->mtu)
            {
                mtuSize = ((CYBLE_GATT_XCHG_MTU_PARAM_T *)eventParam)->mtu;
            }
            else
            {
                mtuSize = CYBLE_GATT_MTU;
            }
            
            break;
            
        case CYBLE_EVT_GATTS_WRITE_REQ:
            
            writeReqParam = (CYBLE_GATTS_WRITE_REQ_PARAM_T *) eventParam;
            
            if(CYBLE_CUSTOM_SERVICE_SENSOR_VALUE_CLIENT_CHARACTERISTIC_CONFIGURATION_DESC_HANDLE== \
                                                                    writeReqParam->handleValPair.attrHandle)
            {
                errorCode = CyBle_GattsWriteAttributeValue(&(writeReqParam->handleValPair), \
                                                0, &cyBle_connHandle, CYBLE_GATT_DB_PEER_INITIATED);
                
                if (CYBLE_GATT_ERR_NONE  == errorCode)
                {
                    #ifdef PRINT_MESSAGE_LOG   
                        UART_UartPutString("\n\rNotifications enabled\n\r");
                        UART_UartPutString("\n\rStart entering data:\n\r");
                    #endif
                    CyBle_GattsWriteRsp(cyBle_connHandle);
                }
            }
              
            break;    
        default:
            break;
    }        
    
    
    switch(advState)
    {
        /* If the bonding is not done and we are trying to connect to any 
         * device for the first time.
         */
        case ADVERTISEMENT_PUBLIC:
            switch(event)
            {
                case CYBLE_EVT_STACK_ON:
                    /* load capacitors on the ECO should be tuned and the tuned value
                    ** must be set in the CY_SYS_XTAL_BLERD_BB_XO_CAPTRIM_REG  */
                    CY_SYS_XTAL_BLERD_BB_XO_CAPTRIM_REG = CAPACITOR_TRIM_VALUE;    
                
                    /* Get the configured clock parameters for BLE sub-system */
                    CyBle_GetBleClockCfgParam(&clockConfig);    
                    
                    /* Update the sleep clock inaccuracy PPM based on WCO crystal used */
                    /* If you see frequent link disconnection, tune your WCO or update the sleep clock accuracy here */
                    clockConfig.bleLlSca =   CYBLE_LL_SCA_000_TO_020_PPM;
                    
                    /* set the clock configuration parameter of BLE sub-system with updated values*/
                    CyBle_SetBleClockCfgParam(&clockConfig);
                    /* Stack initialized; start advertisement */
                    UART_UartPutString("Advertising with public address: ");
                    for(counter = 6; counter > 0; counter--)
                    {
                        UART_UartPutChar(HexToAscii(cyBle_deviceAddress.bdAddr[counter - 1], 1));
                        UART_UartPutChar(HexToAscii(cyBle_deviceAddress.bdAddr[counter - 1], 0));
                        UART_UartPutChar(' ');
                    }
                    AdvertisePublicly();
                    break;
                
                case CYBLE_EVT_GAPP_ADVERTISEMENT_START_STOP:
                    /* If advertisement timed out and we were not connected 
                     * yet, restart advertisement.
                     */
                    if(authState != AUTHENTICATION_BONDING_REMOVE_WAITING_EVENT)
                    {
                        if(CyBle_GetState() == CYBLE_STATE_DISCONNECTED)
                        {
//                            LED_Write(0);
                            CySysPmHibernate();
                        }
                    }
                    else
                    {
                        UART_UartPutString("Advertisement stopped. ");
                        authState = AUTHENTICATION_BONDING_REMOVE_GO_AHEAD;
                    }
                    break;

                case CYBLE_EVT_GAP_DEVICE_DISCONNECTED:
                    /* Disconnected but bonding is not complete; restart 
                     * advertisement.
                     */
                    LEDFLAG=2;
                    UART_UartPutString("\n\n\rDisconnected publicly. ");
                    if(authState != AUTHENTICATION_BONDING_REMOVE_WAITING_EVENT)
                    {
                        UART_UartPutString("Advertising with public address: ");
                        for(counter = 6; counter > 0; counter--)
                        {
                            UART_UartPutChar(HexToAscii(cyBle_deviceAddress.bdAddr[counter - 1], 1));
                            UART_UartPutChar(HexToAscii(cyBle_deviceAddress.bdAddr[counter - 1], 0));
                            UART_UartPutChar(' ');
                        }
                        AdvertisePublicly();
                    }
                    else
                    {
                        authState = AUTHENTICATION_BONDING_REMOVE_GO_AHEAD;
                    }
                    break;
                
                default:
                    break;
            }
            break;
            
            
        /* If the bonding has already happened and we are advertising with 
         * a private address.
         */
        case ADVERTISEMENT_PRIVATE:
            switch(event)
            {
                case CYBLE_EVT_STACK_ON:
                    
                    /* load capacitors on the ECO should be tuned and the tuned value
                    ** must be set in the CY_SYS_XTAL_BLERD_BB_XO_CAPTRIM_REG  */
                    CY_SYS_XTAL_BLERD_BB_XO_CAPTRIM_REG = CAPACITOR_TRIM_VALUE;    
                
                    /* Get the configured clock parameters for BLE sub-system */
                    CyBle_GetBleClockCfgParam(&clockConfig);    
                    
                    /* Update the sleep clock inaccuracy PPM based on WCO crystal used */
                    /* If you see frequent link disconnection, tune your WCO or update the sleep clock accuracy here */
                    clockConfig.bleLlSca =   CYBLE_LL_SCA_000_TO_020_PPM;
                    
                    /* set the clock configuration parameter of BLE sub-system with updated values*/
                    CyBle_SetBleClockCfgParam(&clockConfig);
                    /* Stack initialized; start advertisement */
                    AdvertisePrivately();
                    break;
                    
                case CYBLE_EVT_GAPP_ADVERTISEMENT_START_STOP:
                    /* If advertisement timed out and we were not connected 
                     * yet, restart advertisement.
                     */
                    if(authState != AUTHENTICATION_BONDING_REMOVE_WAITING_EVENT)
                    {
                        if(CyBle_GetState() == CYBLE_STATE_DISCONNECTED)
                        {
                            UART_UartPutString("\n\n\r60 seconds elapsed. ");
                            LED_Write(0);
                            CySysPmHibernate();
                        }
                    }
                    else
                    {
                        UART_UartPutString("Advertisement stopped. ");
                        authState = AUTHENTICATION_BONDING_REMOVE_GO_AHEAD;
                    }
                    break;
                
                case CYBLE_EVT_GAP_DEVICE_DISCONNECTED:
                    /* Disconnected and bonding was previously complete; start 
                     * advertisement with private resolvable address but only 
                     * if the bonding is not to be removed. If the bonding is 
                     * to be removed then advertisement with public address
                     * will be started after the bonding is removed.
                     */
                    
                    
                    UART_UartPutString("\n\n\rDisconnected privately. ");
                    if(authState != AUTHENTICATION_BONDING_REMOVE_WAITING_EVENT)
                    {
                        AdvertisePrivately();
                    }
                    else
                    {
                        authState = AUTHENTICATION_BONDING_REMOVE_GO_AHEAD;
                    }
                    break;
                
                default:
                    break;
            }
            break;
            
            
        default:
            break;
    }
}
__inline void ManageSystemPower()
{
    /* Variable declarations */
    CYBLE_BLESS_STATE_T blePower;
    uint8 interruptStatus ;
    
   /* Disable global interrupts to avoid any other tasks from interrupting this section of code*/
    interruptStatus  = CyEnterCriticalSection();
    
    /* Get current state of BLE sub system to check if it has successfully entered deep sleep mode */
    blePower = CyBle_GetBleSsState();
    
    /* System can enter DeepSleep only when BLESS and rest of the application are in DeepSleep 
     * power modes */
    if((blePower == CYBLE_BLESS_STATE_DEEPSLEEP || blePower == CYBLE_BLESS_STATE_ECO_ON) && 
        applicationPower == DEEPSLEEP)
    {
        applicationPower = WAKEUP_DEEPSLEEP; 
        CySysPmDeepSleep(); 
    }
    else if((blePower != CYBLE_BLESS_STATE_EVENT_CLOSE))
    {
        if(applicationPower == DEEPSLEEP)
        {
            applicationPower = WAKEUP_DEEPSLEEP;
            
            /* change HF clock source from IMO to ECO, as IMO is not required and can be stopped to save power */
            CySysClkWriteHfclkDirect(CY_SYS_CLK_HFCLK_ECO); 
            /* stop IMO for reducing power consumption */
            CySysClkImoStop();              
            /* put the CPU to sleep */
            CySysPmSleep();            
            /* starts execution after waking up, start IMO */
            CySysClkImoStart();
            /* change HF clock source back to IMO */
            CySysClkWriteHfclkDirect(CY_SYS_CLK_HFCLK_IMO);
			
        }
        else if(applicationPower == SLEEP )
        {
            /* If the application requires IMO for its operation, we should not switch the HFCLK source */
            applicationPower = WAKEUP_SLEEP;        
            CySysPmSleep();            
            
        }
    }
    
    /* Enable interrupts */
    CyExitCriticalSection(interruptStatus );

}

/*******************************************************************************
* Function Name: ManageApplicationPower()
********************************************************************************
*
* Summary:
*   This function puts the application componnents in appropriate low power modes 
*   based on the state of the components.
*
* Parameters:
*  none
*
********************************************************************************/
__inline void ManageApplicationPower()
{
    switch(applicationPower)
    {
        case ACTIVE: // dont need to do anything
        break;
        
        case WAKEUP_SLEEP: // do whatever wakeup needs to be done - probably never do anything
        
            applicationPower = ACTIVE;
        break;
        
        case WAKEUP_DEEPSLEEP: // do whatever wakeup needs to be done.
        
            applicationPower = ACTIVE;
        break;
        
        case SLEEP: 
        /** Add code to place the application components to sleep here  */
        break;
        
        case DEEPSLEEP:
        /** Add code to place the application components to deep sleep here  */
        
        break;
    }
}

/*******************************************************************************
* Function Name: RunApplication()
********************************************************************************
*
* Summary:
*   This function runs application specific code. this is a template. This function
*   may be replaced by customer specific functions. 
*
* Parameters:
*  none
*
********************************************************************************/
__inline void RunApplication()
{
    /** Place your application code here **/
    
    /* Once application is done with all processing, enter low-power mode */
    if(0) /* if you are ready to sleep then set it up to go to sleep */
    {
        applicationPower = SLEEP;
    }
    
    if(1) /* if you are done with everything, then deepsleep */
    {
        applicationPower = DEEPSLEEP;
    }
}
void init()
{
    #ifdef SENSOR_ENABLE
    isr_Sensor_StartEx(SensorIsrHandler);
    /* Wait till the switch is pressed */
    while (Sensor_Read() == 0u);//防抖
    CyDelay(100u);

    Sensor_ClearInterrupt();
    isr_Sensor_ClearPending();
    #endif

    WdtIsr_StartEx(WdtIsrHandler);

    #ifdef WAKEUP_ENABLE
        
    isr_WakeupPin_StartEx(WakeupIsrHandler);

    /* Wait till the switch is pressed */
    while (Wakeup_Pin_Read() == 0u);//防抖
    CyDelay(100u);

    Wakeup_Pin_ClearInterrupt();
    isr_WakeupPin_ClearPending();
    
    #endif
    /* Set WDT counter 0 to generate interrupt on match */
	CySysWdtWriteMode(CY_SYS_WDT_COUNTER0, CY_SYS_WDT_MODE_INT);
	CySysWdtWriteMatch(CY_SYS_WDT_COUNTER0, WDT_COUNT0_MATCH);
	CySysWdtWriteClearOnMatch(CY_SYS_WDT_COUNTER0, 1u);
    CySysWdtEnable(CY_SYS_WDT_COUNTER0_MASK);
    CySysWdtLock();
	CySysWdtDisable(CY_SYS_WDT_COUNTER0_MASK);
	CySysWdtUnlock();
    /* Enable global interrupts */
    CyGlobalIntEnable; 
    

    UART_Start();
    
    /* Clear screen and put a welcome message */
    UART_UartPutChar(12);
    UART_UartPutString("========= BLE Privacy Demo - Peripheral =========\n\n\r");
}
/*******************************************************************************
* Function Name: main()
********************************************************************************
* Summary:
* The top level application function.
*
* Parameters:
* None
*
* Return:
* None
*
* Theory:
* Initializes the BLE and UART components and then process BLE events regularly.
* Also decides whether advertisement with public or private address is to be 
* done. Plus it maintains the bonding aspect - whether to add a device to bond
* list or to remove it.
*
* For more information on privacy feature, refer to Bluetooth 4.1 Specification, 
* Volume 3, Part C, Section 10.7.
*
* Side Effects:
* None
*
*******************************************************************************/
int main()
{
    uint8 counter;
    CYBLE_GAP_BD_ADDR_T bondedDeviceAddress;
    CYBLE_GAP_BD_ADDR_T clearAllDevices = {{0,0,0,0,0,0},0};
    uint8 bondedDeviceBdHandle;
    uint8 bondedDeviceKeysFlag;

    #ifdef LOW_POWER_MODE    
    CYBLE_LP_MODE_T         lpMode;
    CYBLE_BLESS_STATE_T     blessState;
    #endif
    init();
        /* Internal low power oscillator is stopped as it is not used in this project */
    CySysClkIloStop();
    
    /* Set the divider for ECO, ECO will be used as source when IMO is switched off to save power */
    CySysClkWriteEcoDiv(CY_SYS_CLK_ECO_DIV8);
    
    /* Start BLE and UART components for the project */
    CyBle_Start(StackEventHandler);
    /* Find out whether the device has bonded information stored already or not */
    CyBle_GapGetBondedDevicesList(&bondedDeviceList);
    if(bondedDeviceList.count != 0)
    {
        UART_UartPutString("restart with privacy address");
        CYBLE_GAP_SMP_KEY_DIST_T securityKeys;
        
        /* Take the last bonded device */
        memcpy(bondedDeviceAddress.bdAddr, bondedDeviceList.bdAddrList[0].bdAddr, 6);
        bondedDeviceAddress.type = bondedDeviceList.bdAddrList[0].type;

        /* Get the BD handle of that bonded device */
        CyBle_GapGetPeerBdHandle(&bondedDeviceBdHandle, &bondedDeviceAddress);
        
        /* Get the IRK for that bonded device */
        bondedDeviceKeysFlag |= 0x01;   /* Identity Resolution Key */ 
        CyBle_GapGetPeerDevSecurityKeyInfo(bondedDeviceBdHandle, &bondedDeviceKeysFlag, &securityKeys);
        memcpy(connectionIrk, securityKeys.irkInfo, CYBLE_GAP_SMP_IRK_SIZE);
        
        /* Start advertisement with private address */
        advState = ADVERTISEMENT_PRIVATE;
        authState = AUTHENTICATION_BONDING_COMPLETE;
    }
    else
    {
        /* Start advertisement with public address */
        advState = ADVERTISEMENT_PUBLIC;
        authState = AUTHENTICATION_NOT_CONNECTED;
    }  

    /* Infinite loop */
    while(1)
    {
        /* Process pending BLE events */

        ManageBlePower();
        /***********************************************************************
        *  Run your application specific code here
        ***********************************************************************/  
//        if(applicationPower == ACTIVE)
//        {
            RunApplication();
//        }
        
        /***********************************************************************
        *  Process application power modes
        ***********************************************************************/
        ManageApplicationPower();   
        
        /***********************************************************************
        *  Manage system power mode based on application and BLE power modes
        ***********************************************************************/
        ManageSystemPower();
        switch(authState)
        {
            case AUTHENTICATION_COMPLETE_BONDING_REQD:
                /* Store bonding data of the current connection */
                while(CYBLE_ERROR_OK != CyBle_StoreBondingData(1));
                
                authState = AUTHENTICATION_BONDING_COMPLETE;
                UART_UartPutString("\n\rBonding complete. Press 'R' at any time to remove this bond. ");
                
                /* Ready for private advertisement the next time */
                advState = ADVERTISEMENT_PRIVATE;
                CyBle_GapGetBondedDevicesList(&bondedDeviceList);
                if(bondedDeviceList.count != 0)
                {
                    UART_UartPutString("bonding success!");
                }
                break;
                
                
            case AUTHENTICATION_BONDING_COMPLETE:
                /* See if the user pressed 'R' button to remove the bond. */
                if((lowPowerFlag==DEBONDING)&&(Sensor_Read()==0))
                {
                        LEDFLAG=0;
                        notifyenable=0;
                        notifyenableH=0;
                        /* User wants the bond to be removed */
                        UART_UartPutString("\n\rRemove the bond. ");
                        if(CyBle_GetState() == CYBLE_STATE_CONNECTED)
                        {
                            /* Disconnect */
                            authState = AUTHENTICATION_BONDING_REMOVE_WAITING_EVENT;
                            CyBle_GapDisconnect(cyBle_connHandle.bdHandle);
                        }
                        else if(CyBle_GetState() == CYBLE_STATE_ADVERTISING)
                        {
                            /* Stop advertisement */
                            authState = AUTHENTICATION_BONDING_REMOVE_WAITING_EVENT;
                            CyBle_GappStopAdvertisement();                            
                        }
                        else
                        {
                            authState = AUTHENTICATION_BONDING_REMOVE_GO_AHEAD;
                        }
                    
                }
                break;

                
            case AUTHENTICATION_BONDING_REMOVE_GO_AHEAD:
                /* Remove all bonded devices in the list */
                CyBle_GapRemoveDeviceFromWhiteList(&clearAllDevices);
                
                while(CYBLE_ERROR_OK != CyBle_StoreBondingData(1));
                UART_UartPutString("Cleared the list of bonded devices. \n\n\r");
                
                /* Clear the IRK used for random private address creation */
                for(counter = 0; counter < CYBLE_GAP_SMP_IRK_SIZE; counter++)
                {
                    connectionIrk[counter] = 0;
                }
                
                /* Start advertising with a public address */
                advState = ADVERTISEMENT_PUBLIC;
                UART_UartPutString("Advertising with public address: ");
                for(counter = 6; counter > 0; counter--)
                {
                    UART_UartPutChar(HexToAscii(cyBle_deviceAddress.bdAddr[counter - 1], 1));
                    UART_UartPutChar(HexToAscii(cyBle_deviceAddress.bdAddr[counter - 1], 0));
                    UART_UartPutChar(' ');
                }
                AdvertisePublicly();
                  
                authState = AUTHENTICATION_NOT_CONNECTED;
                lowPowerFlag=0;
                break;
                
                
            default:
                break;
        }
        
        HandleBleProcessing();
        CyBle_ProcessEvents();

    }
    
}
#ifdef SENSOR_ENABLE
CY_ISR(SensorIsrHandler)
{
    uint8 sensorstate;
    sensorstate=Sensor_Read();
//高低電平發notify，抬手檢測
    /* Debounce delay */
    CyDelay(50u);
    if(sensorstate==(Sensor_Read()))
    {
    /* If the switch is pressed for more than 1 sec, set the flag to enter Stop
     * mode else set the flag to enter hibernate mode. This flag will be cleared
     * by the device when the device wakes up from low power mode
     */
    
    switch(Sensor_Read())
    {
        case 0:
            notifyenable=1;
        break;
        case 1:
            notifyenableH=1;
            
        break;
    }
    }

    Sensor_ClearInterrupt();
    isr_Sensor_ClearPending();
}
#endif


#ifdef WAKEUP_ENABLE


CY_ISR(WakeupIsrHandler)
{
    uint32 counter;

    /* Debounce delay */
    CyDelay(50u);

    /* If the switch is pressed for more than 1 sec, set the flag to enter Stop
     * mode else set the flag to enter hibernate mode. This flag will be cleared
     * by the device when the device wakes up from low power mode
     */
    for(counter = 0u; (counter < 20u) && (Wakeup_Pin_Read() == 0u); counter++)
    {
        CyDelay(50u);
    }

    lowPowerFlag = (counter == 20u) ? DEBONDING : LED_LIGHT_ON;
    if(lowPowerFlag==DEBONDING) LEDFLAG=0;
    if(lowPowerFlag==LED_LIGHT_ON) 
    {
        lowPowerFlag=0;
        LEDFLAG++; 
        if(LEDFLAG==4) LEDFLAG=1;
    }
    Wakeup_Pin_ClearInterrupt();
    isr_WakeupPin_ClearPending();
}
#endif
CY_ISR(WdtIsrHandler)
{
    switch(LEDFLAG)
    {
        case 0:
            LED_Write(0);
            break;
        case 1:
            LED_Write(1);
            break;
        case 2:
            LED_Write(~LED_Read());
            break;
        case 3:
            LED_Write(0);
            break;
        case 4:
            LED_Write(1);
            CyDelay(50);
            LED_Write(0);
            break;

    }
    /* Clear interrupts state */
	CySysWdtClearInterrupt(CY_SYS_WDT_COUNTER0_INT);
    WdtIsr_ClearPending();
}
/* [] END OF FILE */
